/* eslint-disable @typescript-eslint/no-non-null-assertion */
import useRegisterNodeRef from "@utilityjs/use-register-node-ref";
import * as React from "react";
var rootId = 0;
var RootIds = new WeakMap();
var observers = new Map();
var requestIdleCallback = (typeof self !== "undefined" &&
    self.requestIdleCallback &&
    self.requestIdleCallback.bind(window)) ||
    function (cb) {
        var start = Date.now();
        return setTimeout(function () {
            cb({
                didTimeout: false,
                timeRemaining: function () {
                    return Math.max(0, 50 - (Date.now() - start));
                }
            });
        }, 1);
    };
var cancelIdleCallback = (typeof self !== "undefined" &&
    self.cancelIdleCallback &&
    self.cancelIdleCallback.bind(window)) ||
    function (id) {
        return clearTimeout(id);
    };
var getObserver = function () {
    if (typeof global !== "undefined" && global.IntersectionObserver)
        return global.IntersectionObserver;
    if (typeof globalThis !== "undefined" && globalThis.IntersectionObserver)
        return globalThis.IntersectionObserver;
    if (typeof window !== "undefined" && window.IntersectionObserver)
        return window.IntersectionObserver;
    if (typeof IntersectionObserver !== "undefined")
        return IntersectionObserver;
    return null;
};
var getRootId = function (rootOption) {
    if (!rootOption)
        return "0";
    if (RootIds.has(rootOption))
        return RootIds.get(rootOption);
    rootId += 1;
    RootIds.set(rootOption, "" + rootId);
    return "" + rootId;
};
var createId = function (options) {
    return Object.keys(options)
        .sort()
        .filter(function (key) { return options[key] !== undefined; })
        .map(function (key) {
        return key + "_" + (key === "root"
            ? getRootId(options.root)
            : String(options[key]));
    })
        .toString();
};
var observe = function (node, instance) {
    instance.observer.observe(node);
    instance.nodes.push(node);
};
var unobserve = function (node, instance) {
    var index = instance.nodes.indexOf(node);
    if (index > -1)
        instance.nodes.splice(index, 1);
    instance.observer.unobserve(node);
    // Destroy observer when there's nothing left to watch
    if (instance.nodes.length === 0) {
        instance.observer.disconnect();
        observers.delete(instance.id);
    }
};
var createObserver = function (node, callback, options) {
    var Observer = getObserver();
    if (!Observer)
        return null;
    var id = createId(options);
    var instance = observers.get(id);
    if (instance) {
        return {
            observe: function () { return void observe(node, instance); },
            unobserve: function () { return void unobserve(node, instance); }
        };
    }
    var thresholds = { current: [] };
    var observer = new Observer(function (entries) {
        entries.forEach(function (entry) {
            var isIntersected = entry.isIntersecting &&
                thresholds.current.some(function (t) { return entry.intersectionRatio >= t; });
            callback(isIntersected, entry.isIntersecting);
        });
    }, options);
    thresholds.current =
        observer.thresholds ||
            (Array.isArray(options.threshold)
                ? options.threshold
                : [options.threshold]);
    var newInstance = { id: id, observer: observer, nodes: [] };
    observers.set(id, newInstance);
    return {
        observe: function () { return void observe(node, newInstance); },
        unobserve: function () { return void unobserve(node, newInstance); }
    };
};
var useIsInViewport = function (options) {
    var _a = options || {}, _b = _a.threshold, threshold = _b === void 0 ? [0, 1] : _b, _c = _a.root, root = _c === void 0 ? null : _c, _d = _a.rootMargin, rootMargin = _d === void 0 ? "0px" : _d, _e = _a.once, once = _e === void 0 ? false : _e, _f = _a.disabled, disabled = _f === void 0 ? false : _f;
    var _g = React.useState(false), isInViewport = _g[0], setIsInViewport = _g[1];
    var registerNode = useRegisterNodeRef(function (node) {
        if (disabled)
            return;
        var observer = createObserver(node, function (inViewState, isIntersected) {
            if (isIntersected && once)
                observer === null || observer === void 0 ? void 0 : observer.unobserve();
            setIsInViewport(inViewState);
        }, { threshold: threshold, root: root, rootMargin: rootMargin });
        if (!observer)
            return;
        observer.observe();
        return function () { return void observer.unobserve(); };
    }, [disabled]);
    React.useEffect(function () {
        if (getObserver() || isInViewport)
            return;
        var idleCallback = requestIdleCallback(function () { return void setIsInViewport(true); });
        return function () { return cancelIdleCallback(idleCallback); };
    }, [isInViewport]);
    return { registerNode: registerNode, isInViewport: isInViewport };
};
export default useIsInViewport;
